@@ -16,6 +16,8 @@
 """
 
 import unittest
+import sys
+from datetime import datetime
 from test import example_data
 from test.example_models import City, Fighter, Gang, Skill, SkillInstance, FighterSkillList
 from redmodel.models import SetField, ModelWriter, ListFieldWriter, SetFieldWriter, NotFoundError, UniqueError, BadArgsError
@@ -103,15 +105,15 @@ class ModelWriteTestCase(ModelTestCase):
     def test_write(self):
         # basic model
         city_writer = ModelWriter(City)
-        c1 = City(name = 'Reixte')
-        c2 = City(name = 'Damtoo')
-        c3 = City(name = 'Toynbe')
+        c1 = City(name = 'Reixte', coast = True)
+        c2 = City(name = 'Damtoo', coast = True)
+        c3 = City(name = 'Toynbe', coast = False)
         map(city_writer.create, [c1, c2, c3])
         self.assertEqual((c1.oid, c2.oid, c3.oid), ('1', '2', '3'))
         self.assertEqual(City(City.by_id(1)).oid, '1')
-        self.assertEqual(ds.hgetall('City:1'), {'name': 'Reixte'})
-        self.assertEqual(ds.hgetall('City:2'), {'name': 'Damtoo'})
-        self.assertEqual(ds.hgetall('City:3'), {'name': 'Toynbe'})
+        self.assertEqual(ds.hgetall('City:1'), {'name': 'Reixte', 'coast': '1'})
+        self.assertEqual(ds.hgetall('City:2'), {'name': 'Damtoo', 'coast': '1'})
+        self.assertEqual(ds.hgetall('City:3'), {'name': 'Toynbe', 'coast': '0'})
 
         # list field referencing model
         city_connections_writer = ListFieldWriter(City.connections)
@@ -125,19 +127,28 @@ class ModelWriteTestCase(ModelTestCase):
 
         # unique indexed field
         fighter_writer = ModelWriter(Fighter)
-        f1 = Fighter(name = 'Alice', age = 29, weight = 73, city = City.by_id(1))
-        f2 = Fighter(name = 'Bob', age = 32, weight = 98, city = City.by_id(1))
+        dtime1 = datetime.utcfromtimestamp(1400000002)
+        dtime2 = datetime.utcfromtimestamp(1400000001)
+        f1 = Fighter(name = 'Alice', age = 29, weight = 73.2, joined = dtime1, city = City.by_id(1))
+        f2 = Fighter(name = 'Bob', age = 23, weight = 98, joined = dtime2, city = City.by_id(1))
         map(fighter_writer.create, [f1, f2])
+        self.assertEqual(ds.hgetall('Fighter:1'), {'name': 'Alice', 'age': '29', 'weight': '73.2', 'joined': '1400000002', 'city': '1'})
+        self.assertEqual(ds.hgetall('Fighter:2'), {'name': 'Bob', 'age': '23', 'weight': '98', 'joined': '1400000001', 'city': '1'})
         self.assertEqual(ds.hgetall('u:Fighter:name'), {'Alice': '1', 'Bob': '2'})
 
         # indexed reference field
         self.assertEqual(ds.smembers('i:Fighter:city:1'), set(['1', '2']))
 
+        # zindexed fields
+        self.assertEqual(ds.zrange('z:Fighter:age', 0, -1), ['2', '1'])
+        self.assertEqual(ds.zrange('z:Fighter:weight', 0, -1), ['1', '2'])
+        self.assertEqual(ds.zrange('z:Fighter:joined', 0, -1), ['2', '1'])
+
         # missing argument
         self.assertRaises(BadArgsError, Fighter, name = 'MissingWeight', age = 30, city = 1)
 
         # unique attribute
-        f3 = Fighter(name = 'Bob', age = 30, weight = 80, city = 1)
+        f3 = Fighter(name = 'Bob', age = 30, weight = 80, joined = dtime1, city = 1)
         self.assertRaises(UniqueError, fighter_writer.create, f3)
 
         # basic model
@@ -184,7 +195,7 @@ class ModelWriteTestCase(ModelTestCase):
         if __debug__:
             f3skills = FighterSkillList()
             self.assertRaises(AssertionError, fighter_skill_list_writer.create, f3skills)
-            f3 = Fighter(name = 'Unsaved', age = 0, weight = 0, city = 1)
+            f3 = Fighter(name = 'Unsaved', age = 0, weight = 0, joined = None, city = 1)
             self.assertRaises(AssertionError, fighter_skill_list_writer.create, f3skills, f3)
 
         # owned model list field
@@ -219,13 +230,13 @@ class ModelWriteTestCase(ModelTestCase):
         fighter2 = Fighter(Fighter.by_id(2))
         self.assertEqual(fighter2.name, 'Bobby')
         self.assertEqual(fighter2.age, 41)
-        fighter_writer.update(fighter2, name = 'Robert', weight = 99)
+        fighter_writer.update(fighter2, name = 'Robert', weight = 99.9)
         self.assertEqual(ds.hgetall('u:Fighter:name'), {'Alice': '1', 'Robert': '2'})
         self.assertEqual(fighter2.name, 'Robert')
-        self.assertEqual(fighter2.weight, 99)
+        self.assertEqual(fighter2.weight, 99.9)
         fighter3 = Fighter(Fighter.by_id(2))
         self.assertEqual(fighter3.name, 'Robert')
-        self.assertEqual(fighter3.weight, 99)
+        self.assertEqual(fighter3.weight, 99.9)
 
         # update indexed attribute
         self.assertEqual(ds.smembers('i:Fighter:city:1'), set(['1', '2']))
@@ -244,6 +255,13 @@ class ModelWriteTestCase(ModelTestCase):
         self.assertEqual(ds.smembers('i:Fighter:city:1'), set(['1']))
         self.assertEqual(ds.smembers('i:Fighter:city:2'), set(['2']))
 
+        # update zindexed attribute
+        self.assertEqual(ds.zrange('z:Fighter:weight', 0, -1), ['2', '1'])
+        fighter_writer.update(fighter1, weight = 99.91)
+        self.assertEqual(ds.zrange('z:Fighter:weight', 0, -1), ['2', '1'])
+        fighter_writer.update(fighter1, weight = 99.89)
+        self.assertEqual(ds.zrange('z:Fighter:weight', 0, -1), ['1', '2'])
+
     def test_delete(self):
         example_data.load()
 
@@ -251,12 +269,14 @@ class ModelWriteTestCase(ModelTestCase):
         fighter_writer = ModelWriter(Fighter)
         self.assertEqual(ds.hgetall('u:Fighter:name'), {'Alice': '1', 'Bob': '2'})
         self.assertEqual(ds.smembers('i:Fighter:city:1'), set(['1', '2']))
+        self.assertEqual(ds.zrange('z:Fighter:weight', 0, -1), ['2', '1'])
         fighter1 = Fighter(Fighter.by_id(1))
         fighter_writer.delete(fighter1)
         self.assertRaises(NotFoundError, Fighter, Fighter.by_id(1))
         self.assertTrue(fighter1.oid is None)
         self.assertEqual(ds.hgetall('u:Fighter:name'), {'Bob': '2'})
         self.assertEqual(ds.smembers('i:Fighter:city:1'), set(['2']))
+        self.assertEqual(ds.zrange('z:Fighter:weight', 0, -1), ['2'])
 
         # delete container item updates indexes
         gang_members_writer = SetFieldWriter(Gang.members)
@@ -325,6 +345,9 @@ class ModelReadTestCase(ModelTestCase):
         self.assertEqual(fighter1.name, 'Alice')
         self.assertEqual(fighter2.name, 'Bob')
 
+        dtime = datetime.utcfromtimestamp(1400000001)
+        self.assertEqual(fighter2.joined, dtime)
+
         handle1 = Fighter.by_id(1)
         handle2 = Fighter.by_id(2)
         handle3 = Fighter.by_id(1)
@@ -335,10 +358,12 @@ class ModelReadTestCase(ModelTestCase):
 
         city = City(fighter1.city)
         self.assertEqual(city.name, 'Reixte')
+        self.assertEqual(city.coast, True)
         conns = List(city.connections)
         self.assertEqual(len(conns), 2)
         city2 = City(conns[1])
         self.assertEqual(city2.name, 'Toynbe')
+        self.assertEqual(city2.coast, False)
 
         handle = Gang.by_id(999)
         self.assertRaises(NotFoundError, Gang, handle)
@@ -357,10 +382,11 @@ class ModelReadTestCase(ModelTestCase):
         self.assertEqual(handle, Gang.by_id(1))
 
         self.assertTrue(not Gang.find(members__contains = Fighter.by_id(3)))
-        handle = Gang.find(members__contains = hfighter2)
+        handle = Gang.find(members__contains = Fighter(hfighter2))
         self.assertEqual(handle, Gang.by_id(1))
 
-        city_gangs = Gang.multifind(cities__contains = City.by_id(3))
+        city3 = City(City.by_id(3))
+        city_gangs = Gang.multifind(cities__contains = city3)
         self.assertEqual(city_gangs, set([Gang.by_id(1)]))
 
         handle = FighterSkillList.by_owner(fighter1)
@@ -381,6 +407,81 @@ class ModelReadTestCase(ModelTestCase):
         self.assertEqual(skills[1].skill, Skill.by_id(2))
         self.assertEqual(skills[1].value, 91)
 
+        sorted_by_weight_1 = Fighter.zrange('weight')
+        self.assertEqual(sorted_by_weight_1, [hfighter2, hfighter1])
+
+        sorted_by_weight_2 = Fighter.zrange('weight', 0, 0)
+        self.assertEqual(sorted_by_weight_2, [hfighter2])
+
+        sorted_by_weight_3 = Fighter.zrange('weight', 1, -1)
+        self.assertEqual(sorted_by_weight_3, [hfighter1])
+
+        sorted_by_weight_4 = Fighter.zrevrange('weight')
+        self.assertEqual(sorted_by_weight_4, [hfighter1, hfighter2])
+
+        sorted_by_weight_5 = Fighter.zrevrange('weight', 0, 0)
+        self.assertEqual(sorted_by_weight_5, [hfighter1])
+
+        le24a = Fighter.zfind(age__lte = 24)
+        le24b = Fighter.zrangebyscore('age', '-inf', 24)
+        self.assertEqual(le24a, [hfighter1, hfighter2])
+        self.assertEqual(le24a, le24b)
+
+        le23a = Fighter.zfind(age__lte = 23)
+        le23b = Fighter.zrangebyscore('age', '-inf', 23)
+        self.assertEqual(le23a, [hfighter1, hfighter2])
+        self.assertEqual(le23a, le23b)
+
+        lt23a = Fighter.zfind(age__lt = 23)
+        lt23b = Fighter.zrangebyscore('age', '-inf', '(23')
+        self.assertEqual(lt23a, [hfighter1])
+        self.assertEqual(lt23a, lt23b)
+
+        ge24a = Fighter.zfind(age__gte = 24)
+        ge24b = Fighter.zrangebyscore('age', 24, '+inf')
+        self.assertEqual(ge24a, [])
+        self.assertEqual(ge24a, ge24b)
+
+        ge23a = Fighter.zfind(age__gte = 23)
+        ge23b = Fighter.zrangebyscore('age', 23, '+inf')
+        self.assertEqual(ge23a, [hfighter2])
+        self.assertEqual(ge23a, ge23b)
+
+        gt23a = Fighter.zfind(age__gt = 20)
+        gt23b = Fighter.zrangebyscore('age', '(20', '+inf')
+        self.assertEqual(gt23a, [hfighter2])
+        self.assertEqual(gt23a, gt23b)
+
+        age_in_a = Fighter.zfind(age__in = (20, 23))
+        age_in_b = Fighter.zrangebyscore('age', 20, 23)
+        age_in_c = Fighter.zrangebyscore('age', 20, 23, 0, 2)
+        self.assertEqual(age_in_a, [hfighter1, hfighter2])
+        self.assertEqual(age_in_a, age_in_b)
+        self.assertEqual(age_in_a, age_in_c)
+
+        age_eq_a = Fighter.zfind(age = 20)
+        age_eq_b = Fighter.zrangebyscore('age', 20, 20)
+        self.assertEqual(age_eq_a, [hfighter1])
+        self.assertEqual(age_eq_a, age_eq_b)
+
+        joined_before_2014 = Fighter.zfind(joined__lt = datetime(2014, 1, 1))
+        self.assertEqual(joined_before_2014, [])
+        joined_before_2020 = Fighter.zfind(joined__lt = datetime(2020, 1, 1))
+        self.assertEqual(joined_before_2020, [hfighter2, hfighter1])
+        joined_in_201x = Fighter.zfind(joined__in = (datetime(2010, 1, 1), datetime(2020, 1, 1)))
+        self.assertEqual(joined_in_201x, [hfighter2, hfighter1])
+
+        rev_age_in_a = Fighter.zrevrangebyscore('age', 23, 20)
+        rev_age_in_b = Fighter.zrevrangebyscore('age', 23, 20, 0, 2)
+        self.assertEqual(rev_age_in_a, [hfighter2, hfighter1])
+        self.assertEqual(rev_age_in_a, rev_age_in_b)
+
+        self.assertEqual(Fighter.zcount('age', 20, 23), 2)
+        self.assertEqual(Fighter.zrank('weight', fighter1), 1)
+        self.assertEqual(Fighter.zrank('weight', hfighter2), 0)
+        self.assertEqual(Fighter.zrevrank('weight', hfighter1), 0)
+        self.assertEqual(Fighter.zrevrank('weight', hfighter2), 1)
+
 
 def all_tests():
     suite = unittest.TestSuite()
@@ -391,6 +492,8 @@ def all_tests():
 
 
 if __name__ == "__main__":
-    #suite = unittest.TestLoader().loadTestsFromTestCase(Test)
-    suite = all_tests()
-    unittest.TextTestRunner(verbosity=2).run(suite)
+    if len(sys.argv) > 1:
+        unittest.main()
+    else:
+        suite = all_tests()
+        unittest.TextTestRunner(verbosity=2).run(suite)
